-- function that is called on loading the addon
function OnLoad()
	-- register events
	this:RegisterEvent("UNIT_MANA")
	this:SetScript("OnEvent", function(frame, event, firstArg, secondArg)
		if event == "UNIT_MANA" then
			UnitManaHandler(firstArg)
		end
	end)
	-- end: register events
	
	-- register slash commands
	SLASH_MANAWATCH1, SLASH_MANAWATCH2 = "/manawatch", "/mwatch"
	-- end: register slash commands
	
	-- initialize variables
	manaUsedData = {}
	lastMana = UnitMana("player")
	if not AnalyzedTimespan then -- if not set..
		AnalyzedTimespan = 10    -- ..set to default 
	end
	-- end: initialize variables
	
	print("ManaWatch active")
end

-- function that handles the UNIT_MANA event
function UnitManaHandler(unitID)
	if unitID == "player" then
		local currentMana = UnitMana(unitID)
		PlayerManaChanged(currentMana, UnitManaMax(unitID), lastMana - currentMana)
		lastMana = currentMana
	end
end

-- function for calculation of mana usage and output
function PlayerManaChanged(Current, Max, Usage)
	local uptime = GetTime()
	local manaUsed = 0
	manaUsedData[uptime] = Usage
	
	if Current == Max then
		MWMessage:SetText(MW_LOC_FULL_MANA)
		manaUsedData = {}
	else	
		for Time, Mana in pairs(manaUsedData) do
			if uptime - Time > AnalyzedTimespan then
				table.remove(manaUsedData, Time)
			else
				manaUsed = manaUsed + Mana
			end
		end
		
		local avgUsage = manaUsed / AnalyzedTimespan
		local message = "Mana/s: " .. (-avgUsage) .. "|n"
		if avgUsage > 0 then
			local secondsTillOOM = Current / avgUsage
			message = message .. "OOM in " .. string.format("%.0f", secondsTillOOM) .. "s"
			--print("OOM in " .. string.format("%.0f", secondsTillOOM) .. "s") -- DEBUG
		else
			local secondsTillFull = (Max-Current) / -avgUsage
			message = message .. MW_LOC_FULL_MANA .. " in " .. string.format("%.0f", secondsTillFull) .. "s"
			--print("Full Mana in " .. string.format("%.0f", secondsTillFull) .. "s") -- DEBUG
		end
		MWMessage:SetText(message)
	end
end

-- handler for the MANAWATCH slash command
-- /manawatch, /mwatch
function SlashCmdList.MANAWATCH(args, editbox)
	if args == "show" then
		ManaWatchFrame:Show()
	
	elseif args == "hide" then
		ManaWatchFrame:Hide()
	
	elseif string.starts(args, "set timespan ") then
		if (tonumber(string.sub(args, 14))) then
			AnalyzedTimespan = tonumber(string.sub(args, 14))
			print("MW: changed timespan to " .. AnalyzedTimespan)
		end
	
	elseif args == "debug timespan" then
		print("MW: current analyzed timespan: " .. AnalyzedTimespan)
	
	--elseif args == "" then
	--	InterfaceOptionsFrame_OpenToCategory(panel.name);
	
	else -- unknown argument(s)
		print("MW: " .. MW_LOC_UNKNOWN_ARGUMENT .. ": " .. args)
	end
end

-- HELPERS
function string.starts(String,Start)
   return string.sub(String,1,string.len(Start))==Start
end